<?php
/**
* Dynamic To Top Class
*
* @package Dynamic To Top
* @author Matt Varone
*/

if ( !class_exists('Dynamic_To_Top'))
{	
	
	class Dynamic_To_Top
	{
		public $options;

		/** 
		* Dynamic To Top 
		* 
		* Construct class. calls for options and assets.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		*
		*/
		
		function Dynamic_To_Top()
		{
			//	Check if options are stored otherwise populate with defaults.
			if( false === $this->get_options() )
				$this->set_default_options();
			
			// Enqueue CSS and JS.
			$this->enqueue_assets();
		}
		
		/** 
		* Get Options 
		* 
		* Grabs the options from the db and checks for a Dynamic To Top 3.0 
		* specific setting to add an extra validation layer.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return boolean
		*/
		
		function get_options()
		{
			$this->options = get_option('dynamic_to_top');
		
			if ( !empty($this->options) && isset($this->options['padding_top_bottom']) )
				return true;
			else
				return false;
		}
		
		/** 
		* Set Default Options 
		* 
		* Stores default options on the db.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		*/
		
		function set_default_options()
		{
			$this->options = array( 
				'speed' => 1000, 
				'distance' => 200, 
				'easing' => 'In Out',
				'position' => 'Bottom Right',
				'padding_top_bottom' => '21',
				'padding_left_right' => '19',
				'font_size' => '1',
				'text_color' => '#fff',
				'bold'=>'yes',
				'text_shadow'=>'yes',
				'shadow_color'=>'#111',	
				'background_color'=>'#272727',
				'border_color'=>'#000',
				'border_width'=>'1',
				'radius'=>'9',
				'shadow'=>'',
				'inset'=>'yes',
			);
			
			update_option('dynamic_to_top', $this->options);
		}
		
		/** 
		* Enqueue Assets 
		* 
		* Adds JS and CSS assets actions.
		* Checks if its needed to prevent on mobile browsers. 
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		*/
		
		function enqueue_assets()
		{
			if ( $this->is_checked('prevent_on_mobile') )
			{
				if ( $this->is_mobile() )
					return false;
			}
			
			$this->options['margin'] = 20;
			
			add_action('wp_print_styles',  array(&$this,'enqueue_style'));
			add_action('wp_print_scripts', array(&$this,'enqueue_script'));			
		}
		
		/** 
		* Enqueue Style
		* 
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		*/
		
		function enqueue_style()
		{
			wp_enqueue_style( 'dynamic-to-top', DYNAMIC_TO_TOP_URL . '/css/dynamic-to-top-css.php', false, DYNAMIC_TO_TOP_VERSION, 'all' );
		}
		
		/** 
		* Enqueue Script
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		*/
		
		function enqueue_script()
		{
			wp_enqueue_script( 'dynamic-to-top', DYNAMIC_TO_TOP_URL . '/js/dynamic-to-top-js.php', array('jquery'), DYNAMIC_TO_TOP_VERSION, true );			
		}
		
		/** 
		* Get CSS
		*
		* Returns CSS declarations.
		* Checks for transient CSS or generates and set new. 
		* 
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $css css Declarations.
		*/
		
		function get_css($css="")
		{
			$css = get_transient('dynamic_to_top_transient_css');
			
			if ($css)
				return $css;
			
			$css = $this->generate_custom_css();

			set_transient('dynamic_to_top_transient_css',$css,60*60*24*90);
			
			return $css;
		}
		
		/** 
		* Get JS
		*
		* Returns JS declarations.
		* Checks for transient JS or generates and set new. 
		* 
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $js JS scripts.
		*/
		
		function get_js($js="")
		{
			$js = get_transient('dynamic_to_top_transient_js');
			
			if ($js)
				return $js;
			
			$js = $this->generate_custom_js();

			set_transient('dynamic_to_top_transient_js',$js,60*60*24*90);
			
			return $js;
		}
		
		/** 
		* Is Checked 
		* 
		* Conditional method to validate checked options.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @param string $option Option name.
		* @param string $against String to match option.
		* @return boolean
		*/
		
		function is_checked($option="",$against="yes")
		{
			if ($option == "")
				return false;
			
			if (!isset($this->options[$option]))
				return false;
			
			if ($against == "")
				return true;
			else {
				if ($this->options[$option] == $against)
					return true;
				else
					return false;
			} 			
		}
		
		/** 
		* Get Easing Type 
		* 
		* Returns the easing type function name.
		* Used on the JS script.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $easing Name of easing function.
		*/
				
		function get_easing_type($easing="linear")
		{
			switch ($this->options['easing'])
			{
				case 'Bounce':
					$easing = "easeOutBounce";
				break;
				
				case 'Elastic':
					$easing = "easeInElastic";
				break;
				
				case 'In Out':
					$easing = "easeInOutExpo";
				break;
				
				case 'In':
					$easing = "easeInExpo";
				break;
				
				case 'Out':
					$easing = "easeOutExpo";
				break;
			}
			return $easing;
		}
		
		/** 
		* Get Position 
		* 
		* Returns CSS properties for the selected position.
		* Used on the CSS style.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $position Properties of the position selected.
		*/
		
		function get_position($position="")
		{	
			switch ($this->options['position'])
			{
				case 'Bottom Right':
					$position = "bottom:".$this->options['margin']."px;right:".$this->options['margin']."px;top:auto;left:auto;";
				break;
				
				case 'Bottom Left':
					$position = "bottom:".$this->options['margin']."px;left:".$this->options['margin']."px;top:auto;right:auto;";
				break;
				
				case 'Top Right':
					$position = "top:".$this->options['margin']."px;right:".$this->options['margin']."px;bottom:auto;left:auto;";
				break;
				
				case 'Top Left':
					$position = "top:".$this->options['margin']."px;left:".$this->options['margin']."px;bottom:auto;right:auto;";
				break;
			}

			return $position;
		}
		
		/** 
		* Generate Custom CSS 
		* 
		* Returns Dynamic To Top custom CSS styles.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $css CSS declarations.
		*/
		
		function generate_custom_css($css="")
		{
						
			$css .= "/* Dynamic To Top Plugin ver. ".DYNAMIC_TO_TOP_VERSION." - http://www.mattvarone.com */\n\n";
						
			$css .= "body { position:relative}
			
			#dynamic-to-top {
				display:none;
				overflow:hidden;
				width:auto;
				z-index:90;
				position:fixed;
				".$this->get_position()."
								
				font-family: sans-serif;
				font-size:".$this->options['font_size']."em;
				color:".$this->options['text_color'].";
				text-decoration:none;\n";

				if ( $this->is_checked('text_shadow') )
					$css .= "text-shadow:0 1px 0 ".$this->options['shadow_color'].";\n";

				if ( $this->is_checked('bold') )
					$css .="font-weight:bold;\n";
									
			$css.= "padding:".$this->options['padding_top_bottom']."px ".$this->options['padding_left_right']."px;
				border:".$this->options['border_width']."px solid ".$this->options['border_color'].";
				background:".$this->options['background_color'].";
								
				-webkit-background-origin:border;
				-moz-background-origin:border;
				-icab-background-origin:border;
				-khtml-background-origin:border;
				-o-background-origin:border;
				background-origin:border;
				
				-webkit-background-clip:padding-box;
				-moz-background-clip:padding-box;
				-icab-background-clip:padding-box;
				-khtml-background-clip:padding-box;
				-o-background-clip:padding-box;
				background-clip:padding-box;\n";
			
			$box_shadow = "";

			if ( $this->is_checked('shadow')) 
				$box_shadow .= "0 1px 3px rgba(0,0,0,0.4)";

			if ( $this->is_checked('inset') ) {	
				if ( $box_shadow != "") $box_shadow = $box_shadow.",";
				$box_shadow .= "inset 0 0 0 1px rgba(0,0,0,0.2), inset 0 1px 0 rgba(255,255,255,.4),inset 0 10px 10px rgba(255,255,255,.1)";
			}

			if (strlen($box_shadow)>1)
				$css .= "-webkit-box-shadow:".$box_shadow.";
					-ms-box-shadow:".$box_shadow.";
					-moz-box-shadow:".$box_shadow.";
					-o-box-shadow:".$box_shadow.";
					-khtml-box-shadow:".$box_shadow.";
					-icab-box-shadow:".$box_shadow.";
					 box-shadow:".$box_shadow.";\n\n";

			$css .= "-webkit-border-radius: ".$this->options['radius']."px;
					-moz-border-radius: ".$this->options['radius']."px;
					-icab-border-radius: ".$this->options['radius']."px;
					-khtml-border-radius: ".$this->options['radius']."px;
					border-radius: ".$this->options['radius']."px;\n}\n\n";
						
			$css .= "#dynamic-to-top:hover {
				background: ".$this->get_ligther_hex($this->options['background_color']).";
				background: ".$this->options['background_color']." -webkit-gradient( linear, 0% 0%, 0% 100%, from(rgba(255,255,255,.2)),to(rgba(0,0,0,0)));
				background: ".$this->options['background_color']." -webkit-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -khtml-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -moz-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -o-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -ms-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -icab-linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				background: ".$this->options['background_color']."  linear-gradient( top, rgba(255,255,255,.2), rgba(0,0,0,0));
				cursor:pointer;
			}
			
			#dynamic-to-top:active {
				background: ".$this->options['background_color'].";
				background: ".$this->options['background_color']." -webkit-gradient(linear, 0% 0%, 0% 100%, from(rgba(0,0,0,.3)), to(rgba(0,0,0,0)));
				background: ".$this->options['background_color']." -webkit-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -moz-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -khtml-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -o-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -ms-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." -icab-linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));
				background: ".$this->options['background_color']." linear-gradient( top, rgba(0,0,0,.1), rgba(0,0,0,0));								
			}

			#dynamic-to-top,#dynamic-to-top:active,#dynamic-to-top:focus,#dynamic-to-top:hover
			{
				outline:none
			}

			#dynamic-to-top span {
				display:block;
				overflow:hidden;
				width:17px;
				height:12px;
				background:url(".DYNAMIC_TO_TOP_URL."/css/images/up.png) no-repeat center center;
			}";
	
			return $css;
		}
		
		/** 
		* Generate Custom JS script
		* 
		* Returns Dynamic To Top custom JS script.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @return string $js
		*/
		
		function generate_custom_js($js="")
		{
						
			$js .= "/// Dynamic To Top Plugin ver. ".DYNAMIC_TO_TOP_VERSION." - http://www.mattvarone.com\n\n";
						
			$js .= "(function(jQuery){
				jQuery.fn.DynamicToTop = function(options) {

			 		var defaults = {";
			
			if ( $this->is_checked('text_version','yes') )
				$js .= "text: '".$this->options['text']."',";
			else
				$js .= "text: '<span>&nbsp;</span>',";

			$js .="
						min: ".$this->options['distance'].",
						inDelay:600,
						outDelay:400,
			  			containerID: 'dynamic-to-top',
						scrollSpeed: ".$this->options['speed'].",
						easingType: '".$this->get_easing_type()."'
			 		};

			 		var settings = jQuery.extend(defaults, options);
					var containerIDhash = '#'+settings.containerID;

					jQuery('body').append('<a href=\"#\" id=\"'+settings.containerID+'\">'+settings.text+'</a>');

					jQuery(containerIDhash).hide().click(function(){
						jQuery('html, body').animate({scrollTop:0}, settings.scrollSpeed, settings.easingType);
						return false;
					});

					jQuery(window).scroll(function() {
						var sd = jQuery(window).scrollTop();
						if(typeof document.body.style.maxHeight === \"undefined\") {
							jQuery(containerIDhash).css({
								'position': 'absolute',
								'top': jQuery(window).scrollTop() + jQuery(window).height() - ".$this->options['margin']."
							});
						}
						if ( sd > settings.min ) 
							jQuery(containerIDhash).fadeIn(settings.inDelay);
						else 
							jQuery(containerIDhash).fadeOut(settings.Outdelay);
					});

			};
			})(jQuery);\n\n";

			$js .= "// jQuery Easing v1.3 - http://gsgd.co.uk/sandbox/jquery/easing/
			
			if(typeof jQuery.easing.easeInBounce != 'function') { 

			jQuery.extend( jQuery.easing,
			{
				def: 'easeOutExpo',
				swing: function (x, t, b, c, d) {
					return jQuery.easing[jQuery.easing.def](x, t, b, c, d);
				},
				easeInExpo: function (x, t, b, c, d) {
					return (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;
				},
				easeOutExpo: function (x, t, b, c, d) {
					return (t==d) ? b+c : c * (-Math.pow(2, -10 * t/d) + 1) + b;
				},
				easeInOutExpo: function (x, t, b, c, d) {
					if (t==0) return b;
					if (t==d) return b+c;
					if ((t/=d/2) < 1) return c/2 * Math.pow(2, 10 * (t - 1)) + b;
					return c/2 * (-Math.pow(2, -10 * --t) + 2) + b;
				},
				easeInBounce: function (x, t, b, c, d) {
					return c - jQuery.easing.easeOutBounce (x, d-t, 0, c, d) + b;
				},
				easeOutBounce: function (x, t, b, c, d) {
					if ((t/=d) < (1/2.75)) {
						return c*(7.5625*t*t) + b;
					} else if (t < (2/2.75)) {
						return c*(7.5625*(t-=(1.5/2.75))*t + .75) + b;
					} else if (t < (2.5/2.75)) {
						return c*(7.5625*(t-=(2.25/2.75))*t + .9375) + b;
					} else {
						return c*(7.5625*(t-=(2.625/2.75))*t + .984375) + b;
					}
				},
				easeInOutBounce: function (x, t, b, c, d) {
					if (t < d/2) return jQuery.easing.easeInBounce (x, t*2, 0, c, d) * .5 + b;
					return jQuery.easing.easeOutBounce (x, t*2-d, 0, c, d) * .5 + c*.5 + b;
				},
				easeInElastic: function (x, t, b, c, d) {
					var s=1.70158;var p=0;var a=c;
					if (t==0) return b;  if ((t/=d)==1) return b+c;  if (!p) p=d*.3;
					if (a < Math.abs(c)) { a=c; var s=p/4; }
					else var s = p/(2*Math.PI) * Math.asin (c/a);
					return -(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
				},
				easeInOutElastic: function (x, t, b, c, d) {
					var s=1.70158;var p=0;var a=c;
					if (t==0) return b;  if ((t/=d/2)==2) return b+c;  if (!p) p=d*(.3*1.5);
					if (a < Math.abs(c)) { a=c; var s=p/4; }
					else var s = p/(2*Math.PI) * Math.asin (c/a);
					if (t < 1) return -.5*(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
					return a*Math.pow(2,-10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )*.5 + c + b;
				}
			});}\n\n";

			$js .= "\n\njQuery().ready(function() { jQuery('body').DynamicToTop();});\n";			

			return $js;
		}
		
		/** 
		* Is Mobile 
		* 
		* Checks if the browser is mobile.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @author Rever Studios
		* @link http://www.reverbstudios.ie/
		* @return boolean.
		*/		
			
		function is_mobile() 
		{

			if(isset($_SERVER["HTTP_X_WAP_PROFILE"]))
				return true;

			if(preg_match("/wap\.|\.wap/i",$_SERVER["HTTP_ACCEPT"]))
				return true;

			if(isset($_SERVER["HTTP_USER_AGENT"]))
			{
				$user_agents = array(
					"midp", "j2me", "iphone","avantg", "docomo", "novarra", "palmos", 
					"palmsource", "240x320", "opwv", "chtml", "pda", "windows\ ce", "mmp\/", 
					"blackberry", "mib\/", "symbian", "wireless", "nokia", "hand", "mobi", 
					"phone", "cdm", "up\.b", "audio", "SIE\-", "SEC\-", "samsung", "HTC", 
					"mot\-", "mitsu", "sagem", "sony", "alcatel", "lg", "erics", "vx", "NEC", 
					"philips", "mmm", "xx", "panasonic", "sharp", "wap", "sch", "rover", 
					"pocket", "benq", "java", "pt", "pg", "vox", "amoi", "bird", "compal", 
					"kg", "voda", "sany", "kdd", "dbt", "sendo", "sgh", "gradi", "jb", "\d\d\di", "moto");
					
				foreach($user_agents as $user_string)
				{
					if(preg_match("/".$user_string."/i",$_SERVER["HTTP_USER_AGENT"])) 
						return true;
				}
			}

			return false;
		}
		
		/** 
		* Get Lighter Hex 
		* 
		* Returns a lighter version of an Hex color.
		*
		* @package Dynamic To Top
		* @subpackage Main Class
		* @since 3.0
		* @author Justin French
		* @link http://justinfrench.com/
		* @return boolean.
		*/
		
		function get_ligther_hex($hex,$factor = 30)
		{
	
			$new_hex = '#'; 

			$base['R'] = hexdec($hex{0}.$hex{1}); 
			$base['G'] = hexdec($hex{2}.$hex{3}); 
			$base['B'] = hexdec($hex{4}.$hex{5}); 

			foreach ($base as $k => $v) 
			{ 
				$amount = 255 - $v; 
				$amount = $amount / 100; 
				$amount = round($amount * $factor); 
				$new_decimal = $v + $amount; 

				$new_hex_component = dechex($new_decimal); 
				
				if(strlen($new_hex_component) < 2) 
				$new_hex_component = "0".$new_hex_component;
			        
				$new_hex .= $new_hex_component; 
			}
			
			return $new_hex;
		}
		
	
	}
	
	/*
	|--------------------------------------------------------------------------
	| DYNAMIC TO TOP CLASS INITIALIZE
	|--------------------------------------------------------------------------
	*/

	if (!function_exists('dynamic_to_top_init'))
	{

		/** 
		* Initializes the main Dynamic to top class.
		*
		* @package Dynamic To Top
		* @since 3.0
		*
		*/

		function dynamic_to_top_init()
		{
			global $OBJ_dynamic_to_top;
			$OBJ_dynamic_to_top = new Dynamic_To_Top();
		}

		add_action('init', 'dynamic_to_top_init');
	}
}